Class: Exception::Fault::ImplicitlySystem

Inherits:
System
  • Object
show all
Defined in:
lib/cli/kit/core_ext.rb

Overview

The default fault for exceptions that have not been explicitly attributed. Nothing should reference this constant directly — it exists only as the default return value of Exception#fault. Use rubocop to enforce this.

Inherits from System so that fault.is_a?(System) and checks like bug? (which uses Fault::System ===) treat unattributed errors as system errors by default.

Instance Method Summary collapse

Instance Method Details

#to_sObject

: -> String



37
# File 'lib/cli/kit/core_ext.rb', line 37

def to_s = 'implicitly_system'