Module: CLI::Kit::Levenshtein
- Defined in:
- lib/cli/kit/levenshtein.rb
Class Method Summary collapse
-
.distance(str1, str2) ⇒ Object
This code is based directly on the Text gem implementation Copyright (c) 2006-2013 Paul Battley, Michael Neumann, Tim Fletcher.
-
.min3(a, b, c) ⇒ Object
detects the minimum value out of three arguments.
Class Method Details
.distance(str1, str2) ⇒ Object
This code is based directly on the Text gem implementation Copyright (c) 2006-2013 Paul Battley, Michael Neumann, Tim Fletcher.
Returns a value representing the "cost" of transforming str1 into str2 : (String str1, String str2) -> Integer
38 39 40 41 42 43 44 45 46 47 48 49 50 51 52 53 54 55 56 57 58 59 60 61 62 63 64 65 66 67 68 69 70 |
# File 'lib/cli/kit/levenshtein.rb', line 38 def distance(str1, str2) n = str1.length m = str2.length return m if n.zero? return n if m.zero? d = (0..m).to_a x = 0 # to avoid duplicating an enumerable object, create it outside of the loop str2_codepoints = str2.codepoints str1.each_codepoint.with_index(1) do |char1, i| j = 0 while j < m cost = char1 == str2_codepoints[j] ? 0 : 1 a = d[j] #: as !nil b = d[j + 1] #: as !nil x = min3( b + 1, i + 1, # deletion a + cost, # substitution ) d[j] = i i = x j += 1 end d[m] = x end x end |
.min3(a, b, c) ⇒ Object
detects the minimum value out of three arguments. This method is
faster than [a, b, c].min and puts less GC pressure.
See https://github.com/yuki24/did_you_mean/pull/1 for a performance
benchmark.
: (Integer a, Integer b, Integer c) -> Integer
80 81 82 83 84 85 86 87 88 |
# File 'lib/cli/kit/levenshtein.rb', line 80 def min3(a, b, c) if a < b && a < c a elsif b < c b else c end end |